@@ -1,4 +1,4 @@
-lo = function(rown, coln, nrow, ncol, cellheight = NA, cellwidth = NA, treeheight_col, treeheight_row, legend, annotation, annotation_colors, annotation_legend, fontsize, fontsize_row, fontsize_col, ...){
+lo = function(rown, coln, nrow, ncol, cellheight = NA, cellwidth = NA, treeheight_col, treeheight_row, legend, annotation, annotation_colors, annotation_legend, main, fontsize, fontsize_row, fontsize_col, ...){
 	# Get height of colnames and length of rownames
 	if(!is.null(coln[1])){
 		longest_coln = which.max(nchar(coln))
@@ -31,6 +31,14 @@ lo = function(rown, coln, nrow, ncol, cellheight = NA, cellwidth = NA, treeheigh
 		legend_width = unit(0, "bigpts")
 	}
 	
+	# Set main title height
+	if(is.na(main)){
+		main_height = unit(0, "npc")
+	}
+	else{
+		main_height = unit(1.5, "grobheight", textGrob(main, gp = gpar(fontsize = 1.3 * fontsize, ...)))
+	}
+	
 	# Column annotations
 	if(!is.na(annotation[[1]][1])){
 		# Column annotation height 
@@ -60,19 +68,20 @@ lo = function(rown, coln, nrow, ncol, cellheight = NA, cellwidth = NA, treeheigh
 	}
 	
 	if(is.na(cellheight)){
-		matheight = unit(1, "npc") - coln_height - treeheight_col - annot_height
+		matheight = unit(1, "npc") - main_height - coln_height - treeheight_col - annot_height
 	}
 	else{
 		matheight = unit(cellheight * nrow, "bigpts")
 	}	
 	
+	
 	# Produce layout()
-	pushViewport(viewport(layout = grid.layout(nrow = 4, ncol = 5, widths = unit.c(treeheight_row, matwidth, rown_width, legend_width, annot_legend_width), heights = unit.c(treeheight_col, annot_height, matheight, coln_height))))
+	pushViewport(viewport(layout = grid.layout(nrow = 5, ncol = 5, widths = unit.c(treeheight_row, matwidth, rown_width, legend_width, annot_legend_width), heights = unit.c(main_height, treeheight_col, annot_height, matheight, coln_height)), gp = do.call(gpar, gp)))
 	
 	# Get cell dimensions
-	pushViewport(vplayout(3, 2))
+	pushViewport(vplayout(4, 2))
 	cellwidth = convertWidth(unit(0:1, "npc"), "bigpts", valueOnly = T)[2] / ncol
-	cellheight = convertHeight(unit(0:1, "npc"), "bigpts", valueOnly = T)[2] / ncol
+	cellheight = convertHeight(unit(0:1, "npc"), "bigpts", valueOnly = T)[2] / nrow
 	upViewport()
 	
 	# Return minimal cell dimension in bigpts to decide if borders are drawn
@@ -157,7 +166,7 @@ convert_annotations = function(annotation, annotation_colors){
 	for(i in 1:ncol(annotation)){
 		a = annotation[, i]
 		b = annotation_colors[[colnames(annotation)[i]]]
-		if(class(a) %in% c("character", "factor")){
+		if(is.character(a) | is.factor(a)){
 			a = as.character(a)
 			if(length(setdiff(names(b), a)) > 0){
 				stop(sprintf("Factor levels on variable %s do not match with annotation_colors", colnames(annotation)[i]))
@@ -188,7 +197,7 @@ draw_annotation_legend = function(annotation, annotation_colors, border_color, .
 	for(i in names(annotation_colors)){
 		grid.text(i, x = 0, y = y, vjust = 1, hjust = 0, gp = gpar(fontface = "bold", ...))
 		y = y - 1.5 * text_height
-		if(class(annotation[, i]) %in% c("character", "factor")){
+		if(is.character(annotation[, i]) | is.factor(annotation[, i])){
 			for(j in 1:length(annotation_colors[[i]])){
 				grid.rect(x = unit(0, "npc"), y = y, hjust = 0, vjust = 1, height = text_height, width = text_height, gp = gpar(col = border_color, fill = annotation_colors[[i]][j]))
 				grid.text(names(annotation_colors[[i]])[j], x = text_height * 1.3, y = y, hjust = 0, vjust = 1, gp = gpar(...))
@@ -208,42 +217,52 @@ draw_annotation_legend = function(annotation, annotation_colors, border_color, .
 	}
 }
 
+draw_main = function(text, ...){
+	grid.text(text, gp = gpar(fontface = "bold", ...))
+}
 
 vplayout = function(x, y){
 	return(viewport(layout.pos.row = x, layout.pos.col = y))
 }
 
-heatmap_motor = function(matrix, border_color, cellwidth, cellheight, tree_col, tree_row, treeheight_col, treeheight_row, filename, width, height, breaks, color, legend, annotation, annotation_colors, annotation_legend, fontsize, fontsize_row, fontsize_col, ...){
+heatmap_motor = function(matrix, border_color, cellwidth, cellheight, tree_col, tree_row, treeheight_col, treeheight_row, filename, width, height, breaks, color, legend, annotation, annotation_colors, annotation_legend, main, fontsize, fontsize_row, fontsize_col, ...){
 	grid.newpage()
 	
 	# Set layout
-	mindim = lo(coln = colnames(matrix), rown = rownames(matrix), nrow = nrow(matrix), ncol = ncol(matrix), cellwidth = cellwidth, cellheight = cellheight, treeheight_col = treeheight_col, treeheight_row = treeheight_row, legend = legend, annotation = annotation, annotation_colors = annotation_colors, annotation_legend = annotation_legend, fontsize = fontsize, fontsize_row = fontsize_row, fontsize_col = fontsize_col,  ...)
+	mindim = lo(coln = colnames(matrix), rown = rownames(matrix), nrow = nrow(matrix), ncol = ncol(matrix), cellwidth = cellwidth, cellheight = cellheight, treeheight_col = treeheight_col, treeheight_row = treeheight_row, legend = legend, annotation = annotation, annotation_colors = annotation_colors, annotation_legend = annotation_legend, main = main, fontsize = fontsize, fontsize_row = fontsize_row, fontsize_col = fontsize_col,  ...)
 	
 	# Omit border color if cell size is too small 
 	if(mindim < 3) border_color = NA
 	
+	# Draw title
+	if(!is.na(main)){
+		pushViewport(vplayout(1, 2))
+		draw_main(main, fontsize = 1.3 * fontsize, ...)
+		upViewport()
+	}
+	
 	# Draw tree for the columns
 	if(!is.na(tree_col[[1]][1]) & treeheight_col != 0){
-		pushViewport(vplayout(1, 2))
+		pushViewport(vplayout(2, 2))
 		draw_dendrogram(tree_col, horizontal = T)
 		upViewport()
 	}
 	
 	# Draw tree for the rows
 	if(!is.na(tree_row[[1]][1]) & treeheight_row != 0){
-		pushViewport(vplayout(3, 1))
+		pushViewport(vplayout(4, 1))
 		draw_dendrogram(tree_row, horizontal = F)
 		upViewport()
 	}
 	
 	# Draw matrix
-	pushViewport(vplayout(3, 2))
+	pushViewport(vplayout(4, 2))
 	draw_matrix(matrix, border_color)
 	upViewport()
 	
 	# Draw colnames
 	if(length(colnames(matrix)) != 0){
-		pushViewport(vplayout(4, 2))
+		pushViewport(vplayout(5, 2))
 		pars = list(colnames(matrix), fontsize = fontsize_col, ...)
 		do.call(draw_colnames, pars)
 		upViewport()
@@ -251,7 +270,7 @@ heatmap_motor = function(matrix, border_color, cellwidth, cellheight, tree_col,
 	
 	# Draw rownames
 	if(length(rownames(matrix)) != 0){
-		pushViewport(vplayout(3, 3))
+		pushViewport(vplayout(4, 3))
 		pars = list(rownames(matrix), fontsize = fontsize_row, ...)
 		do.call(draw_rownames, pars)
 		upViewport()
@@ -259,7 +278,7 @@ heatmap_motor = function(matrix, border_color, cellwidth, cellheight, tree_col,
 	
 	# Draw annotation tracks
 	if(!is.na(annotation[[1]][1])){
-		pushViewport(vplayout(2, 2))
+		pushViewport(vplayout(3, 2))
 		converted_annotation = convert_annotations(annotation, annotation_colors)
 		draw_annotations(converted_annotation, border_color)
 		upViewport()
@@ -267,19 +286,19 @@ heatmap_motor = function(matrix, border_color, cellwidth, cellheight, tree_col,
 	
 	# Draw annotation legend
 	if(!is.na(annotation[[1]][1]) & annotation_legend){
-		pushViewport(vplayout(3, 5))
+		pushViewport(vplayout(3:5, 5))
 		draw_annotation_legend(annotation, annotation_colors, border_color, fontsize = fontsize, ...)
 		upViewport()
 	}
 	
 	# Draw legend
 	if(!is.na(legend[1])){
-		pushViewport(vplayout(3, 4))
+		pushViewport(vplayout(3:5, 4))
 		draw_legend(color, breaks, legend, fontsize = fontsize, ...)
 		upViewport()
 	}
 	
-	pushViewport(vplayout(1:4, 1:5))
+	pushViewport(vplayout(1:5, 1:5))
 	
 	if(!is.na(filename)){
 		if(is.na(height)){
@@ -306,7 +325,7 @@ heatmap_motor = function(matrix, border_color, cellwidth, cellheight, tree_col,
 		
 		# print(sprintf("height:%f width:%f", height, width))
 		f(filename, height = height, width = width)
-		heatmap_motor(matrix, cellwidth = cellwidth, cellheight = cellheight, border_color = border_color, tree_col = tree_col, tree_row = tree_row, treeheight_col = treeheight_col, treeheight_row = treeheight_row, breaks = breaks, color = color, legend = legend, annotation = annotation, annotation_colors = annotation_colors, annotation_legend = annotation_legend, filename = NA, fontsize = fontsize, fontsize_row = fontsize_row, fontsize_col = fontsize_col, ...)
+		heatmap_motor(matrix, cellwidth = cellwidth, cellheight = cellheight, border_color = border_color, tree_col = tree_col, tree_row = tree_row, treeheight_col = treeheight_col, treeheight_row = treeheight_row, breaks = breaks, color = color, legend = legend, annotation = annotation, annotation_colors = annotation_colors, annotation_legend = annotation_legend, filename = NA, main = main, fontsize = fontsize, fontsize_row = fontsize_row, fontsize_col = fontsize_col, ...)
 		dev.off()
 	}
 }
@@ -367,7 +386,7 @@ generate_annotation_colours = function(annotation, annotation_colors){
 	}
 	count = 0
 	for(i in 1:ncol(annotation)){
-		if(class(annotation[, i]) %in% c("character", "factor")){
+		if(is.character(annotation[, i]) | is.factor(annotation[, i])){
 			count = count + length(unique(annotation[, i]))
 		}
 	}
@@ -379,7 +398,7 @@ generate_annotation_colours = function(annotation, annotation_colors){
 	
 	for(i in 1:ncol(annotation)){
 		if(!(colnames(annotation)[i] %in% names(annotation_colors))){
-			if(class(annotation[, i]) %in% c("character", "factor")){
+			if(is.character(annotation[, i]) | is.factor(annotation[, i])){
 				ind = sample(1:length(factor_colors), length(unique(annotation[, i])))
 				annotation_colors[[colnames(annotation)[i]]] = factor_colors[ind]
 				l = levels(as.factor(annotation[, i]))
@@ -397,14 +416,37 @@ generate_annotation_colours = function(annotation, annotation_colors){
 	return(annotation_colors)
 }
 
-
+kmeans_pheatmap = function(mat, k = min(nrow(mat), 150), sd_limit = NA, ...){
+	# Filter data
+	if(!is.na(sd_limit)){
+		s = apply(mat, 1, sd)
+		mat = mat[s > sd_limit, ]	
+	}
+	
+	# Cluster data
+	km = kmeans(mat, k, iter.max = 100)
+	mat2 = km$centers
+	
+	# Compose rownames
+	t = table(km$cluster)
+	rownames(mat2) = sprintf("cl%s_size_%d", names(t), t)
+	
+	# Draw heatmap
+	pheatmap(mat2, ...)
+}
  
 #' A function to draw clustered heatmaps.
 #' 
-#' Detailed description of function
+#' The function also allows to aggregate the rows using kmeans clustering. This is 
+#' advisable if number of rows is so big that R cannot handle their hierarchical 
+#' clustering anymore, roughly more than 1000. Instead of showing all the rows 
+#' separately one can cluster the rows in advance and show only the cluster centers. 
+#' The number of clusters can be tuned with parameter kmeans_k.
 #'
 #' @param mat numeric matrix of the values to be plotted.
 #' @param color vector of colors used in heatmap.
+#' @param kmeans_k the number of kmeans clusters to make, if we want to agggregate the 
+#' rows before drawing heatmap. If NA then the rows are not aggregated.
 #' @param breaks a sequence of numbers that covers the range of values in mat and is one 
 #' element longer than color vector. Used for mapping values to colors. Useful, if needed 
 #' to map certain values to certain colors, to certain values. If value is NA then the 
@@ -444,6 +486,7 @@ generate_annotation_colours = function(annotation, annotation_colors){
 #' should be drawn. 
 #' @param show_rownames boolean specifying if column names are be shown.
 #' @param show_colnames boolean specifying if column names are be shown.
+#' @param main the title of the plot
 #' @param fontsize base fontsize for the plot 
 #' @param fontsize_row fontsize for rownames (Default: fontsize) 
 #' @param fontsize_col fontsize for colnames (Default: fontsize) 
@@ -466,11 +509,12 @@ generate_annotation_colours = function(annotation, annotation_colors){
 #'
 #'	# Draw heatmaps
 #'	pheatmap(test)
+#'  pheatmap(test, kmeans_k = 2)
 #'	pheatmap(test, scale = "row", clustering_distance_rows = "correlation")
 #'	pheatmap(test, color = colorRampPalette(c("navy", "white", "firebrick3"))(50))
 #'	pheatmap(test, cluster_row = FALSE)
 #'	pheatmap(test, legend = FALSE)
-#'	pheatmap(test, cellwidth = 15, cellheight = 12)
+#'	pheatmap(test, cellwidth = 15, cellheight = 12, main = "Example heatmap")
 #'	pheatmap(test, cellwidth = 15, cellheight = 12, fontsize = 8, filename = "test.pdf")
 #'
 #'
@@ -489,7 +533,7 @@ generate_annotation_colours = function(annotation, annotation_colors){
 #'
 #'	ann_colors = list(Var1 = Var1, Var2 = Var2)
 #'
-#'	pheatmap(test, annotation = annotation, annotation_colors = ann_colors)
+#'	pheatmap(test, annotation = annotation, annotation_colors = ann_colors, main = "Example with all the features")
 #' 
 #'	# Specifying clustering from distance matrix
 #'	drows = dist(test, method = "minkowski")
@@ -497,12 +541,23 @@ generate_annotation_colours = function(annotation, annotation_colors){
 #'	pheatmap(test, clustering_distance_rows = drows, clustering_distance_cols = dcols)
 #' 
 #' @export
-pheatmap = function(mat, color = colorRampPalette(rev(c("#D73027", "#FC8D59", "#FEE090", "#FFFFBF", "#E0F3F8", "#91BFDB", "#4575B4")))(100), breaks = NA, border_color = "grey60", cellwidth = NA, cellheight = NA, scale = "none", cluster_rows = TRUE, cluster_cols = TRUE, clustering_distance_rows = "euclidean", clustering_distance_cols = "euclidean", clustering_method = "complete",  treeheight_row = ifelse(cluster_rows, 50, 0), treeheight_col = ifelse(cluster_cols, 50, 0), legend = TRUE, annotation = NA, annotation_colors = NA, annotation_legend = TRUE, show_rownames = T, show_colnames = T, fontsize = 10, fontsize_row = fontsize, fontsize_col = fontsize, filename = NA, width = NA, height = NA, ...){
+pheatmap = function(mat, color = colorRampPalette(rev(c("#D73027", "#FC8D59", "#FEE090", "#FFFFBF", "#E0F3F8", "#91BFDB", "#4575B4")))(100), kmeans_k = NA, breaks = NA, border_color = "grey60", cellwidth = NA, cellheight = NA, scale = "none", cluster_rows = TRUE, cluster_cols = TRUE, clustering_distance_rows = "euclidean", clustering_distance_cols = "euclidean", clustering_method = "complete",  treeheight_row = ifelse(cluster_rows, 50, 0), treeheight_col = ifelse(cluster_cols, 50, 0), legend = TRUE, annotation = NA, annotation_colors = NA, annotation_legend = TRUE, show_rownames = T, show_colnames = T, main = NA, fontsize = 10, fontsize_row = fontsize, fontsize_col = fontsize, filename = NA, width = NA, height = NA, ...){
 	
 	# Preprocess matrix
 	mat = as.matrix(mat)
 	mat = scale_mat(mat, scale)
 	
+	# Kmeans
+	if(!is.na(kmeans_k)){
+		# Cluster data
+		km = kmeans(mat, kmeans_k, iter.max = 100)
+		mat = km$centers
+
+		# Compose rownames
+		t = table(km$cluster)
+		rownames(mat) = sprintf("cl%s_size_%d", names(t), t)
+	}
+	
 	# Do clustering
 	if(cluster_rows){
 		tree_row = cluster_mat(mat, distance = clustering_distance_rows, method = clustering_method)
@@ -549,7 +604,7 @@ pheatmap = function(mat, color = colorRampPalette(rev(c("#D73027", "#FC8D59", "#
 	}
 	
 	# Draw heatmap
-	heatmap_motor(mat, border_color = border_color, cellwidth = cellwidth, cellheight = cellheight, treeheight_col = treeheight_col, treeheight_row = treeheight_row, tree_col = tree_col, tree_row = tree_row, filename = filename, width = width, height = height, breaks = breaks, color = color, legend = legend, annotation = annotation, annotation_colors = annotation_colors, annotation_legend = annotation_legend, fontsize = fontsize, fontsize_row = fontsize_row, fontsize_col = fontsize_col, ...)
+	heatmap_motor(mat, border_color = border_color, cellwidth = cellwidth, cellheight = cellheight, treeheight_col = treeheight_col, treeheight_row = treeheight_row, tree_col = tree_col, tree_row = tree_row, filename = filename, width = width, height = height, breaks = breaks, color = color, legend = legend, annotation = annotation, annotation_colors = annotation_colors, annotation_legend = annotation_legend, main = main, fontsize = fontsize, fontsize_row = fontsize_row, fontsize_col = fontsize_col, ...)
 }
 
 #' Pretty heatmaps
