@@ -6,9 +6,11 @@ roxygenize(package.dir = "Source", roxygen.dir = "pheatmap", unlink.target = T,
 
 ## Run in terminal
 cd ~/Raivo/Projects/RHeatmap/
+rm -r pheatmap/.git
+rm -r pheatmap/inst
 R CMD check pheatmap
 R CMD build pheatmap
-R CMD install pheatmap_0.5.tar.gz 
+R CMD install pheatmap_0.5.3.tar.gz 
 
 
 # Debug, kui kisab et dokument sisaldab mitte ascii tähti
